//
//  TVOCallInvite.h
//  TwilioVoice
//
//  Copyright © 2018 Twilio, Inc. All rights reserved.
//

@class TVOAcceptOptions;
@class TVOCall;
@protocol TVOCallDelegate;

/**
 * The `TVOCallInvite` object represents an incoming Call Invite. `TVOCallInvite`s are not created directly;
 * they are returned by the `<[TVONotificationDelegate callInviteReceived:]>` delegate method.
 */
@interface TVOCallInvite : NSObject

/**
 * @name Properties
 */

/**
 * @brief `From` value of the Call Invite.
 *
 * @discussion This may be `nil` if the notification passed in `<[TwilioVoice handleNotification:delegate:]>`
 * method does not have valid information in it.
 */
@property (nonatomic, copy, readonly, nullable) NSString *from;

/**
 * @brief `To` value of the Call Invite.
 */
@property (nonatomic, copy, readonly, nonnull) NSString *to;

/**
 * @brief A server assigned identifier (SID) for the incoming Call.
 *
 * @discussion Accepting a `TVOCallInvite` yields a `TVOCall` which inherits this SID.
 *
 * @see TVOCall.sid
 */
@property (nonatomic, copy, readonly, nonnull) NSString *callSid;

/**
 * @brief Custom parameters embedded in the VoIP notification payload.
 *
 * @discussion The custom parameters will be received in the notification payload with the
 * `twi_params` key and in query-string format, e.g. `key1=value1&key2=value2`. To receive custom
 * parameters on the mobile client, add `<Parameter>` tags into the `<Client>` tag in the TwiML response.
 *
 *   <?xml version="1.0" encoding="UTF-8"?>
 *   <Response>
 *     <Dial callerId="client:alice">
 *       <Client>
 *         <Identity>bob</Identity>
 *         <Parameter name="caller_first_name" value="alice" />
 *         <Parameter name="caller_last_name" value="smith" />
 *       </Client>
 *     </Dial>
 *   </Response>
 *
 * The `customParameters` value would be:
 *
 *   {
 *       "caller_first_name" = "alice";
 *       "caller_last_name" = "smith";
 *   }
 *
 * Note: While the value field passed into `<Parameter>` gets URI encoded by the Twilio infrastructure
 * and URI decoded when parsed during the creation of a `TVOCallInvite`, the name does not get URI encoded
 * or decoded. As a result, it is recommended that the name field only use ASCII characters.
 */
@property (nonatomic, strong, readonly, nullable) NSDictionary<NSString *, NSString *> *customParameters;

/**
 * @name Call Invite Actions
 */

/**
 * @brief Accepts the incoming Call Invite.
 *
 * @param delegate A `<TVOCallDelegate>` to receive Call state updates.
 * @return A `TVOCall` object.
 *
 * @see TVOCallDelegate
 */
- (nonnull TVOCall *)acceptWithDelegate:(nonnull id<TVOCallDelegate>)delegate;

/**
 * @brief Accepts the incoming Call Invite.
 *
 * @param options An accept options.
 * @param delegate A `<TVOCallDelegate>` to receive Call state updates.
 *
 * @return A `TVOCall` object.
 *
 * @see TVOAcceptOptions
 * @see TVOCallDelegate
 */
- (nonnull TVOCall *)acceptWithOptions:(nonnull TVOAcceptOptions *)options delegate:(nonnull id<TVOCallDelegate>)delegate;

/**
 * @brief Rejects the incoming Call Invite.
 */
- (void)reject;

/**
 * @brief Call Invites cannot be instantiated directly. See `<TVONotificationDelegate>` instead.
 *
 * @see TVONotificationDelegate
 */
- (null_unspecified instancetype)init __attribute__((unavailable("Call Invites cannot be instantiated directly. See `TVONotificationDelegate`")));

@end


/**
 *  CallKit specific additions.
 */
@interface TVOCallInvite (CallKit)

/**
 * @brief UUID of the Call Invite.
 *
 * @discussion Use this UUID for CallKit methods. Accepting a `TVOCallInvite` yields a `TVOCall` which inherits its
 * UUID from the Invite.
 */
@property (nonatomic, copy, readonly, nonnull) NSUUID *uuid;

@end
